/*
 * ar4c-dcdc - io-read.c
 *
 * Copyright (C) 2014  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <compiler.h>
#include <leds.h>
#include <comm.h>
#include <adc.h>
#include <tio.h>
#include <ltypes.h>

#define ADC_VBAT		0
#define ADC_VIN			1

#define R1		39.0e3
#define R2		510.0e3
#define VREF		2.63
#define ADC_RANGE	((R1 + R2) / R1 * VREF)
#define C_SHIFT		15
#define IN_SCALER	(u16)((ADC_RANGE / 65.536) * (1UL << C_SHIFT))
#define IN_SCALER_MIN	(u16)((unsigned long) IN_SCALER * 9 / 10)
#define IN_SCALER_MAX	(u16)((unsigned long) IN_SCALER * 11 / 10)

static u16 scaler;

static int scale_adc(unsigned int x)
{
	return (scaler * (unsigned long)x) >> C_SHIFT;
}

void tio_read(void)
{
	unsigned int vin;
	unsigned int vbat;

	adc_enable();
	vin = adc_get(ADC_VIN);
	vbat = adc_get(ADC_VBAT);
	adc_disable();

	comm_reg_write(COMM_VIN_RAW, vin);
	vin = scale_adc(vin);
	comm_reg_write(COMM_VIN, vin);

	comm_reg_write(COMM_VBAT_RAW, vbat);
	vbat = scale_adc(vbat);
	comm_reg_write(COMM_VBAT, vbat);
}

void __init tio_init(void)
{
	u16 x;

	x = comm_reg_read(COMM_SCALER);
	if (!x)
		x = IN_SCALER;
	else if (x < IN_SCALER_MIN)
		x = IN_SCALER_MIN;
	else if (x > IN_SCALER_MAX)
		x = IN_SCALER_MAX;
	comm_reg_write(COMM_SCALER, x);
	scaler = x;
}
